# Copyright © 2023 pkeorley
#
# This source code is licensed under the MIT license found in the LICENSE
# file in the root directory of this source tree.

from typing import Literal

import lastfmxpy.config as config
from lastfmxpy.exeptions import LFPPanic
from lastfmxpy.params.base_classes import BaseArtistParams
from lastfmxpy.typings import Autocorrect
from lastfmxpy.utils import Utils


class AddTags(BaseArtistParams):
    def __init__(
            self,
            artist: str,
            tags: list[str],
            api_key: str = None,
            api_sig: str = None,
            sk: str = None
    ):
        """
        Tag an artist with one or more user supplied tags.
        :param artist: The artist name
        :param tags: List of user supplied tags to apply to this artist. Accepts a maximum of 10 tags.
        :param api_key: A Last.fm API key.
        :param api_sig: A Last.fm method signature. See authentication (https://www.last.fm/api/authentication) for more information.
        :param sk: A session key generated by authenticating a user via the authentication protocol.
        """
        super().__init__()

        self.artist = artist

        if len(tags) > config.MAX_COUNT_OF_TAGS:
            raise LFPPanic(f"Accepts a maximum of 10 tags. You have {len(tags)}")

        self.tags = tags
        self.api_key = api_key
        self.api_sig = api_sig
        self.sk = sk

    def to_json(self) -> dict:
        return Utils.remove_none_values({
            "artist": self.artist,
            "tags": ",".join(self.tags),
            "api_key": self.api_key,
            "api_sig": self.api_sig,
            "sk": self.sk
        })


class GetCorrection(BaseArtistParams):
    def __init__(
            self,
            artist: str,
            api_key: str = None
    ):
        super().__init__()

        self.artist = artist
        self.api_key = api_key

    def to_json(self) -> dict:
        return Utils.remove_none_values({
            "artist": self.artist,
            "api_key": self.api_key
        })


class GetInfo(BaseArtistParams):
    def __init__(
            self,
            artist: str,
            mbid: str = None,
            lang: str = None,
            autocorrect: Literal[0, 1] = None,
            username: str = None,
            api_key: str = None
    ):
        """
        Get the metadata for an artist. Includes biography, truncated at 300 characters.
        :param artist: The artist name
        :param mbid: The musicbrainz id for the artist
        :param lang: The language to return the biography in, expressed as an ISO 639 alpha-2 code.
        :param autocorrect: Transform misspelled artist names into correct artist names, returning the correct version instead. The corrected artist name will be returned in the response.
        :param username: The username for the context of the request. If supplied, the user's playcount for this artist is included in the response.
        :param api_key: A Last.fm API key.
        """
        super().__init__()

        if not any((
                artist,
                mbid
        )):
            raise LFPPanic("You must meet one of these conditions: fill in 'artist' OR specify 'mbid'")

        self.artist = artist
        self.mbid = mbid
        self.lang = lang
        self.autocorrect = autocorrect
        self.username = username
        self.api_key = api_key

    def to_json(self) -> dict:
        return Utils.remove_none_values({
            "artist": self.artist,
            "mbid": self.mbid,
            "lang": self.lang,
            "autocorrect": self.autocorrect,
            "username": self.username,
            "api_key": self.api_key
        })


class GetSimilar(BaseArtistParams):
    def __init__(
            self,
            artist: str,
            limit: int = None,
            autocorrect: Autocorrect = None,
            mbid: str = None,
            api_key: str = None
    ):
        """
        Get all the artists similar to this artist
        :param artist: The artist name
        :param limit: Limit the number of similar artists returned
        :param autocorrect: Transform misspelled artist names into correct artist names, returning the correct version instead. The corrected artist name will be returned in the response.
        :param mbid: The musicbrainz id for the artist
        :param api_key: A Last.fm API key.
        """
        super().__init__()

        self.artist = artist
        self.limit = limit
        self.autocorrect = autocorrect
        self.mbid = mbid
        self.api_key = api_key

    def to_json(self) -> dict:
        return Utils.remove_none_values({
            "artist": self.artist,
            "limit": self.limit,
            "autocorrect": self.autocorrect,
            "mbid": self.mbid,
            "api_key": self.api_key
        })


class GetTags(BaseArtistParams):
    def __init__(
            self,
            artist: str = None,
            mbid: str = None,
            user: str = None,
            autocorrect: Autocorrect = None,
            api_key: str = None
    ):
        """
        Get the tags applied by an individual user to an artist on Last.fm. If accessed as an authenticated service /and/ you don't supply a user parameter then this service will return tags for the authenticated user. To retrieve the list of top tags applied to an artist by all users use artist.getTopTags.
        :param artist: The artist name
        :param mbid: The musicbrainz id for the artist
        :param user:  If called in non-authenticated mode you must specify the user to look up
        :param autocorrect: Transform misspelled artist names into correct artist names, returning the correct version instead. The corrected artist name will be returned in the response.
        :param api_key: A Last.fm API key.
        """

        super().__init__()

        if not any((
                artist,
                mbid
        )):
            raise LFPPanic("You must meet one of these conditions: fill in 'artist' OR specify 'mbid'")

        self.artist = artist
        self.mbid = mbid
        self.user = user
        self.autocorrect = autocorrect
        self.api_key = api_key

    def to_json(self) -> dict:
        return Utils.remove_none_values({
            "artist": self.artist,
            "mbid": self.mbid,
            "user": self.user,
            "autocorrect": self.autocorrect,
            "api_key": self.api_key
        })


class GetTopAlbums(BaseArtistParams):
    def __init__(
            self,
            artist: str = None,
            mbid: str = None,
            autocorrect: Autocorrect = None,
            page: str | int = None,
            limit: str | int = None,
            api_key: str = None
    ):
        """
        Get the top albums for an artist on Last.fm, ordered by popularity.
        :param artist: The artist name
        :param mbid: The musicbrainz id for the artist
        :param autocorrect: Transform misspelled artist names into correct artist names, returning the correct
        :param page: The page number to fetch. Defaults to first page.
        :param limit: The number of results to fetch per page. Defaults to 50.
        :param api_key: A Last.fm API key.
        """
        super().__init__()

        if not any((
                artist,
                mbid
        )):
            raise LFPPanic("You must meet one of these conditions: fill in 'artist' OR specify 'mbid'")

        self.artist = artist
        self.mbid = mbid
        self.autocorrect = autocorrect
        self.page = page
        self.limit = limit
        self.api_key = api_key

    def to_json(self) -> dict:
        return Utils.remove_none_values({
            "artist": self.artist,
            "mbid": self.mbid,
            "autocorrect": self.autocorrect,
            "page": self.page,
            "limit": self.limit,
            "api_key": self.api_key
        })


class GetTopTags(BaseArtistParams):
    def __init__(
            self,
            artist: str = None,
            mbid: str = None,
            autocorrect: Autocorrect = None,
            api_key: str = None
    ):
        """
        Get the top tags for an artist on Last.fm, ordered by popularity.
        :param artist: The artist name
        :param mbid: The musicbrainz id for the artist
        :param autocorrect: Transform misspelled artist names into correct artist names, returning the correct version instead. The corrected artist name will be returned in the response.
        :param api_key: A Last.fm API key.
        """
        super().__init__()
        if not any((
                artist,
                mbid
        )):
            raise LFPPanic("You must meet one of these conditions: fill in 'artist' OR specify 'mbid'")

        self.artist = artist
        self.mbid = mbid
        self.autocorrect = autocorrect
        self.api_key = api_key

    def to_json(self) -> dict:
        return Utils.remove_none_values({
            "artist": self.artist,
            "mbid": self.mbid,
            "autocorrect": self.autocorrect,
            "api_key": self.api_key
        })


class GetTopTracks(BaseArtistParams):
    def __init__(
            self,
            artist: str = None,
            mbid: str = None,
            autocorrect: Autocorrect = None,
            page: str | int = None,
            limit: str | int = None,
            api_key: str = None
    ):
        """
        Get the top tracks by an artist on Last.fm, ordered by popularity
        :param artist: The artist name
        :param mbid: The musicbrainz id for the artist
        :param autocorrect: Transform misspelled artist names into correct artist names, returning the correct
        :param page: The page number to fetch. Defaults to first page.
        :param limit: The number of results to fetch per page. Defaults to 50.
        :param api_key: A Last.fm API key.
        """
        super().__init__()

        if not any((
                artist,
                mbid
        )):
            raise LFPPanic("You must meet one of these conditions: fill in 'artist' OR specify 'mbid'")

        self.artist = artist
        self.mbid = mbid
        self.autocorrect = autocorrect
        self.page = page
        self.limit = limit
        self.api_key = api_key

    def to_json(self) -> dict:
        return Utils.remove_none_values({
            "artist": self.artist,
            "mbid": self.mbid,
            "autocorrect": self.autocorrect,
            "page": self.page,
            "limit": self.limit,
            "api_key": self.api_key
        })


class RemoveTag(BaseArtistParams):
    def __init__(
            self,
            artist: str,
            tag: str,
            api_key: str = None,
            api_sig: str = None,
            sk: str = None
    ):
        """
        Remove a user's tag from an artist.
        :param artist: The artist name
        :param tag: A single user tag to remove from this artist.
        :param api_key: A Last.fm API key.
        :param api_sig: A Last.fm method signature. See authentication for more information.
        :param sk: A session key generated by authenticating a user via the authentication protocol.
        """
        super().__init__()

        self.artist = artist
        self.tag = tag
        self.api_key = api_key
        self.api_sig = api_sig
        self.sk = sk

    def to_json(self) -> dict:
        return Utils.remove_none_values({
            "artist": self.artist,
            "tag": self.tag,
            "api_key": self.api_key,
            "api_sig": self.api_sig,
            "sk": self.sk
        })


class Search(BaseArtistParams):
    def __init__(
            self,
            artist: str,
            limit: str | int = None,
            page: str | int = None,
            api_key: str = None
    ):
        """
        Search for an artist by name. Returns artist matches sorted by relevance.
        :param artist: The artist name
        :param limit: The number of results to fetch per page. Defaults to 30.
        :param page: The page number to fetch. Defaults to first page.
        :param api_key: A Last.fm API key.
        """
        super().__init__()

        self.artist = artist
        self.limit = limit
        self.page = page
        self.api_key = api_key

    def to_json(self) -> dict:
        return Utils.remove_none_values({
            "artist": self.artist,
            "limit": self.limit,
            "page": self.page,
            "api_key": self.api_key
        })
